==> cetus-5.3.5.tgz: package/validator/index.js <==
"use strict";
let Promise         =   require('bluebird'),
    _               =   require('underscore'),
    sanitizer       =   require('sanitizer'),
    utilities       =   require('../utilities/index'),
    config          =   {}, //require('./config'),
    tests           =   {},
    validationTest  =   {};


// return regex validator function
function createValidationTest (criteria) { return value => config.regex[criteria].test (value); }

// return string matching function
//function createStringMatchTest (criteria) { return value => _.indexOf(config.stringSets[criteria], value) > -1 ? true : false; }
function createStringMatchTest (criteria) { return value => config.stringSets[criteria].findIndex (string => string === value) > -1;  }

// return object validator function
function createObjectValidationTest (criteria) {
    let returnFunction = [ '(function (value) {' ,
        'if ( !_.isObject(value) ) return false;',
        'if ( exports.array (value) ) return false;',
        'if ( exports.string (value) ) return false;',
        'return ('
    ];

    _.each(config.objects[criteria], function(value, key){
        if (value[0].required) returnFunction.push( 'exports[\'' + value[0].test + '\'](value.' + key + ') &&' );
        else returnFunction.push( '( _.isUndefined (value.' + key + ') ? true : exports[\'' + value[0].test + '\'](value.' + key + ') ) &&' );
    });

    returnFunction.push (' true ) })');
    return eval(returnFunction.join(''));
}

// return function that checks an array of 'criteria' if each element is valid
function createValidationTestOnArray (criteria) {
    if ( !criteria || !exports[criteria] ) return function () { utilities.logger.warn (criteria + ': no such validation function')};
    return function (value) {
        if ( !exports.array (value) )       return false;
        if ( !exports.notEmpty (value) )    return false;
        return _.every(value, exports[criteria]);
     };
}

function sanitizeObject (data, callback) {
    _.each (data, (element, index) => exports.sanitize (element).then(result => data[index] = result ));
    callback(data);
};

function isPropertyInvalid (data, property) {
    return _.find (config.rules[property], (rule) => {
        let ruleTest = !exports[rule]( data [property] );
        if (ruleTest) {
            utilities.logger.warn (property + ' property did not pass the ' + rule + ' test.');
            if (property !== 'password') utilities.logger.warn (data);
        }
        return ruleTest;
    });
}

function arePropertiesInvalid (data, properties) {
    return _.find (properties, (property) => {
        if (!config.rules[property]) {
            utilities.logger.warn ('Valid rules do not exist for ' + property + ' property!');
            return true;        // return invalid
        }
        return isPropertyInvalid(data, property);
    });
}

//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  {
    if (options && _.isObject(options)) _.extend(config, options);

    //build list of string matchers based on String match returned function
    _.each (config.stringSets, (value, key) => exports[key] = createStringMatchTest (key) );

    //build list of regex validators based on regex validator returned function
    _.each (config.regex,      (value, key) => exports[key] = createValidationTest (key) );

    //build list of function validators
    _.each (config.functions,  (value, key) => exports[key] = value );

    //build list of object validators
    _.each (config.objects,    (value, key) => exports[key + 'Object'] = createObjectValidationTest (key) );

    //build list of array validators
    _.each (config.arrays,     (element, index) => exports[element + 'Array'] = createValidationTestOnArray (element) );
};

exports.missingProperties = function (obj, propsArray)  {
    let missing = _.reject( propsArray, prop => _.has(obj, prop) );
    if (missing.length) utilities.logger.warn ('Missing properties: ' + missing );
    return missing.length;
};

exports.isValid = function (data, properties) {
    if ( _.isUndefined(data) || _.isNull(data) || _.isFunction(data) || !_.isObject(data) || _.isEmpty(data) ) return false;
    if ( _.isUndefined(properties) || _.isNull(properties) || _.isFunction(properties) || !_.isArray(properties) || _.isEmpty(properties) ) return false;

    let notValid = _.find (properties, (property) => {
        if (!config.rules[property]) {
            utilities.logger.warn ('Valid rules do not exist for ' + property + ' property!'); return true;
        }
        let notValidAgain = _.find (config.rules[property], (rule) => {
            let ruleTest = !exports[rule]( data [property] );
            if (ruleTest) {
                utilities.logger.warn (property + ' property did not pass the ' + rule + ' test.');
                if (property !== 'password') utilities.logger.warn (data);
            }
            return ruleTest;
        });
        return !_.isUndefined(notValidAgain);
    });
    return !notValid;
};

exports.isValidPromise = function (data, properties) {
    return new Promise ( (resolve,reject) => {
        if (!properties.length) return resolve(data);
        if ( _.isUndefined(data) || _.isNull(data) || _.isFunction(data) || !_.isObject(data) || _.isEmpty(data) ) return reject(new utilities.error.app(351));
        if ( _.isUndefined(properties) || _.isNull(properties) || _.isFunction(properties) || !_.isArray(properties) ) return reject(new utilities.error.app(351));
        if ( arePropertiesInvalid (data, properties)) return reject(new utilities.error.app(351));
        return resolve(data);
    });
};

exports.sanitize = function (data) {
    return new Promise ( (resolve,reject) => {
        if (data.email) data.email = data.email.toLowerCase(); // change emails to lowercase
        if (_.isString(data)) return resolve(sanitizer.unescapeEntities(sanitizer.sanitize(data)));
        else if (_.isFunction(data)) return resolve(null);
        else if (_.isObject(data)) sanitizeObject (data, resolve);
        else return resolve(data);
    });
};

==> cetus-8.0.30.tgz: package/validator/index.js <==
"use strict";
let Promise         =   require('bluebird'),
    _               =   require('underscore'),
    sanitizer       =   require('sanitizer'),
    utilities       =   require('../utilities/index'),
    config          =   {}, //require('./config'),
    tests           =   {},
    validationTest  =   {};


// return regex validator function
function createValidationTest (criteria) { return value => config.regex[criteria].test (value); }

// return string matching function
//function createStringMatchTest (criteria) { return value => _.indexOf(config.stringSets[criteria], value) > -1 ? true : false; }
function createStringMatchTest (criteria) { return value => config.stringSets[criteria].findIndex (string => string === value) > -1;  }

// return object validator function
function createObjectValidationTest (criteria) {
    let returnFunction = [ '(function (value) {' ,
        'if ( !_.isObject(value) ) return false;',
        'if ( exports.array (value) ) return false;',
        'if ( exports.string (value) ) return false;',
        'return ('
    ];

    _.each(config.objects[criteria], function(value, key){
        if (value[0].required) returnFunction.push( 'exports[\'' + value[0].test + '\'](value.' + key + ') &&' );
        else returnFunction.push( '( _.isUndefined (value.' + key + ') ? true : exports[\'' + value[0].test + '\'](value.' + key + ') ) &&' );
    });

    returnFunction.push (' true ) })');
    return eval(returnFunction.join(''));
}

// return function that checks an array of 'criteria' if each element is valid
function createValidationTestOnArray (criteria) {
    if ( !criteria || !exports[criteria] ) return function () { utilities.logger.warn (criteria + ': no such validation function')};
    return function (value) {
        if ( !exports.array (value) )       return false;
        if ( !exports.notEmpty (value) )    return false;
        return _.every(value, exports[criteria]);
     };
}

/*function sanitizeObject (data, callback) {
    _.each (data, (element, index) => exports.sanitize (element).then(result => data[index] = result ));
    callback(data);
};*/

function sanitizeObject (data) {
    _.each (data, (element, index) => exports.sanitize (element).then(result => data[index] = result ));
    return data;
};

function isPropertyInvalid (data, property) {
    let error = false
    _.find (config.rules[property], (rule) => {
        const doesRuleFail = !exports[rule](data[property]);
        if (doesRuleFail) {
            error = new utilities.error.app(351, `${property} (${data[property]}) did not pass the ${rule} test`);
        }
        return doesRuleFail;
    });

    return error;
}

function arePropertiesInvalid (data, properties) {
    let error = false;
    
    const badProperty = _.find (properties, property => {
        if (!config.rules[property]) {
            error = new utilities.error.app(200, `Valid rules do not exist for ${property} property!`);
            return true;
        } else {
            error = isPropertyInvalid(data, property);
        }
    });

    return error;
}

//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  {
    if (options && _.isObject(options)) _.extend(config, options);

    //build list of string matchers based on String match returned function
    _.each (config.stringSets, (value, key) => exports[key] = createStringMatchTest (key) );

    //build list of regex validators based on regex validator returned function
    _.each (config.regex,      (value, key) => exports[key] = createValidationTest (key) );

    //build list of function validators
    _.each (config.functions,  (value, key) => exports[key] = value );

    //build list of object validators
    _.each (config.objects,    (value, key) => exports[key + 'Object'] = createObjectValidationTest (key) );

    //build list of array validators
    _.each (config.arrays,     (element, index) => exports[element + 'Array'] = createValidationTestOnArray (element) );
};

exports.missingProperties = (obj, propsArray) => _.reject(propsArray, prop => _.has(obj, prop));

exports.isValid = function (data, properties) {
    if ( _.isUndefined(data) || _.isNull(data) || _.isFunction(data) || !_.isObject(data) || _.isEmpty(data) ) return false;
    if ( _.isUndefined(properties) || _.isNull(properties) || _.isFunction(properties) || !_.isArray(properties) || _.isEmpty(properties) ) return false;

    let notValid = _.find (properties, property => {
        if (!config.rules[property]) {
            utilities.logger.error (`Valid rules do not exist for ${property} property!`); 
            return true;
        }
        let notValidAgain = _.find (config.rules[property], rule => {
            let ruleTest = !exports[rule]( data[property] );
            if (ruleTest) {
                utilities.logger.error (`${property} (${data[property]}) did not pass the ${rule} test`);
            }
            return ruleTest;
        });
        return !_.isUndefined(notValidAgain);
    });
    return !notValid;
};

exports.isValidPromise = function (data, properties) {
    if (!properties.length) {
        return Promise.resolve(data);
    }

    if (_.isUndefined(data) || _.isNull(data) || _.isFunction(data) || !_.isObject(data) || _.isEmpty(data)) {
        return Promise.reject(new utilities.error.app(200, "Data is not defined."));
    }

    if (_.isUndefined(properties) || _.isNull(properties) || _.isFunction(properties) || !_.isArray(properties)) {
        return Promise.reject(new utilities.error.app(200, "Properties are not defined."));
    }

    const propertiesAreInvalid = arePropertiesInvalid(data, properties);
    
    return propertiesAreInvalid ? Promise.reject(propertiesAreInvalid) : Promise.resolve(data);
};

exports.sanitize = function (data) {
    if (data.email) {
        data.email = data.email.toLowerCase(); // change emails to lowercase
    }
    
    if (_.isString(data)) {
        return Promise.resolve(sanitizer.unescapeEntities(sanitizer.sanitize(data)));
    } else if (_.isFunction(data)) {
        return Promise.resolve(null);
    } else if (_.isObject(data)) {
        return Promise.resolve(sanitizeObject(data));
    } else {
        return Promise.resolve(data);
    } 
};

/*exports.sanituze = function (data, callback) {
    return new Promise ( (resolve,reject) => {
        if (data.email) data.email = data.email.toLowerCase(); // change emails to lowercase
        if (_.isString(data)) return resolve(sanitizer.unescapeEntities(sanitizer.sanitize(data)));
        else if (_.isFunction(data)) return resolve(null);
        else if (_.isObject(data)) sanitizeObject (data, resolve);
        else return resolve(data);
    });
};*/
